 
Library ieee;		   
use ieee.std_logic_1164.all;		   
use ieee.numeric_std.all;		   
package tcds_types is		   

constant fw_major : integer := 3;	 -- version	   
constant fw_minor : integer := 4;	 -- revision		   
constant fw_build : integer := 3;	 -- build	 	   
constant fw_year	: integer := 14;	 -- 2014		   
constant fw_month : integer := 12;	 -- July (7)		   
constant fw_day	: integer := 09;	 -- day	   

constant fw_id		: std_logic_vector(31 downto 0) :=	std_logic_vector(to_unsigned(	fw_major	,4)) &
																		std_logic_vector(to_unsigned(	fw_minor	,4)) &
																		std_logic_vector(to_unsigned(	fw_build	,8)) &
																		std_logic_vector(to_unsigned(	fw_year	,7)) &
																		std_logic_vector(to_unsigned(	fw_month	,4)) &
																		std_logic_vector(to_unsigned(	fw_day	,5));

constant system_id	: std_logic_vector(31 downto 0) := X"54434453";	-- ASCII "TCDS"		   

constant ttype_reserved_0		: std_logic_vector(3 downto 0) := X"0";--X"F";
constant ttype_global			: std_logic_vector(3 downto 0) := X"1";--X"1";
constant ttype_sequence			: std_logic_vector(3 downto 0) := X"2";--X"2";
constant ttype_unbiased			: std_logic_vector(3 downto 0) := X"3";--X"3";
constant ttype_auxiliary		: std_logic_vector(3 downto 0) := X"4";--X"3";
constant ttype_reserved_5		: std_logic_vector(3 downto 0) := X"5";--X"F";
constant ttype_reserved_6		: std_logic_vector(3 downto 0) := X"6";--X"F";
constant ttype_reserved_7		: std_logic_vector(3 downto 0) := X"7";--X"F";
constant ttype_cyclic			: std_logic_vector(3 downto 0) := X"8";--X"3";
constant ttype_bunch				: std_logic_vector(3 downto 0) := X"9";--X"3";
constant ttype_soft				: std_logic_vector(3 downto 0) := X"A";--X"3";
constant ttype_tts				: std_logic_vector(3 downto 0) := X"B";--X"3";
constant ttype_reserved_c		: std_logic_vector(3 downto 0) := X"C";--X"F";
constant ttype_reserved_d		: std_logic_vector(3 downto 0) := X"D";--X"F";
constant ttype_reserved_e		: std_logic_vector(3 downto 0) := X"E";--X"F";
constant ttype_reserved_f		: std_logic_vector(3 downto 0) := X"F";--X"F";

constant	bgo_luminibble					: std_logic_vector(5 downto 0) := "00" & x"0";
constant	bgo_bc0							: std_logic_vector(5 downto 0) := "00" & x"1";
constant	bgo_testenable					: std_logic_vector(5 downto 0) := "00" & x"2";
constant	bgo_privategap					: std_logic_vector(5 downto 0) := "00" & x"3";
constant	bgo_privateorbit				: std_logic_vector(5 downto 0) := "00" & x"4";
constant	bgo_Resynch						: std_logic_vector(5 downto 0) := "00" & x"5";
constant	bgo_HardReset					: std_logic_vector(5 downto 0) := "00" & x"6";
constant	bgo_Event_counter_reset		: std_logic_vector(5 downto 0) := "00" & x"7";
constant	bgo_Orbit_counter_reset		: std_logic_vector(5 downto 0) := "00" & x"8";
constant	bgo_Start						: std_logic_vector(5 downto 0) := "00" & x"9";
constant	bgo_Stop							: std_logic_vector(5 downto 0) := "00" & x"A";
constant	bgo_startofgap					: std_logic_vector(5 downto 0) := "00" & x"B";
constant	bgo_reserved12					: std_logic_vector(5 downto 0) := "00" & x"C";
constant	bgo_warningtestenable		: std_logic_vector(5 downto 0) := "00" & x"D";
constant	bgo_reserved14					: std_logic_vector(5 downto 0) := "00" & x"E";
constant	bgo_reserved15					: std_logic_vector(5 downto 0) := "00" & x"F";

constant	bgo_lumi_nibble_section_reset		: std_logic_vector(5 downto 0) := bgo_Orbit_counter_reset;


constant lhc_bunch_count : integer := 3564;
constant number_of_lpms : integer := 12;		   
type AMC_hss_data_array is array(1 to number_of_lpms)  of std_logic_vector(15 downto 0);		   
constant max_number_of_tts_channels : integer := 256;	-- used in bgo_train entity

constant ici_address_width : integer := 18;		   
constant apve_address_width : integer := 15;		   
constant lumi_address_width : integer := 14;		   
constant NumberOfRegs : integer := 512;		   
constant IntAddrWidth : integer := 18;		   
--constant ttcnum :integer := 16;		   
constant n_o_bchannels : integer := 16;		   
constant n_o_bgos : integer := 64;
--assert not(n_o_bchannels > max_n_o_bchannels) report "FAILURE: Maximum number of Bchannels exceeded" severity failure;
--max_bch_error: assert not(n_o_bchannels > max_n_o_bchannels) severity failure;
constant n_o_bgo_bits : integer := 6;		   
constant lumidaq_record_size : integer := 256 + 64;--  -- 32 bit words
constant bst_record_size : integer := 8*68; -- (64 bytes) bits in Beam Synchronous Timing (BST) packet

--max_n_o_bgo_train_definitions_error: assert not(n_o_bgo_train_definitions > max_n_o_bgo_train_definitions) severity failure;

-- ramdef_array specifies the Bchannels that impelments a large RAM;
-- Only test_enable (Bgo#2) and Warning_Test_Enable (#13) need large RAM.
constant ramdef_array : std_logic_vector(n_o_bchannels - 1 downto 0) := std_logic_vector(to_unsigned(4 + 8192, n_o_bchannels));   

-- registers per iCi instance:
constant BRD_ID :integer := 0;	 -- Board type (read only)	   
constant SYS_ID :integer := 1;	 -- System ID (read only)	   
constant FWARE_VER : integer := 2;	 -- Firmware version (read only)	   
constant TRIG_NUM : integer := 3;	 -- number of cyclic trigger generators (b0-7; level0, b8-15; b-go) (read only)	   
constant EVNCNT : integer := 4;	 -- Event counter value (read only)	   
constant ORBCNT : integer := 5;	 -- Orbit counter value (read only)	   
constant STRCNT : integer := 6;	 -- Strobe counter value (read only)	   
constant STATUS : integer := 7;	 -- Board status (read only)	   
constant CONTROL : integer := 8;	 -- global control register; contains trigger delays
constant LASTEVTORBCNT : integer := 9;	 -- Orbit counter value at latest trigger (read only)	   
constant TRG_POST : integer := 10;	 -- Level 1 trigger post-scale	   
constant BUNCH_TRGGEN_PRE : integer := 11;	 -- Bunch trigger pre-scale	   
constant DAQ_CONFIG : integer := 12;	 -- pipeline length, buffer depths	   
constant BGO_TRAIN_BLOCK_ON_TTS : integer := 13;	 -- pipeline length, buffer depths	   
constant SEMAPHORE : integer := 15;	 -- Semaphore register	   
constant CNTRST : integer := 16;	 -- TTCci, CTG, CBG Counters Resets	   
constant BGO_MAP : integer := 17;	 -- no longer used; exist in the Ci	   
constant INSELECT : integer := 18;	 -- Front Panel Input Selection	   
--constant OUTSELECT : integer := 19;	 -- B-Channel Output Data Selection	   
constant QPLLCTRL : integer := 20;	 -- QPLL Control Signals	   
constant ORBOFFS : integer := 21;	 -- Orbit Offset setting for LEMO input (used in iPM and iCi)	   
constant ORBOFFS2 : integer := 22;	 -- Orbit Offset setting for LTC and CTC input (used in iCi)	   
constant RNDTRG : integer := 23;	 -- Random trigger fraction of 32 bits; X"00800000" gives 1/512 of 40MHz, thus 78 kHz
constant VMEBGO : integer := 24;	 -- VME Initiated B-Go Input, used as    
constant ipb_datS : integer := 25;	 -- VME Asynchronous Data Short Word	   
constant ipb_datL : integer := 26;	 -- VME Asynchronous Data Long Word	   
constant BDAT_CFG : integer := 27;	 -- Bdata configuration	   
constant BDAT_SPY : integer := 28;	 -- Bdata spy; contains the most recent bdata word sent	   
constant TTS_TRIGGER_CONFIG : integer := 29;	-- TTS based action trigger configuration   
constant BGO_TRAIN_CONFIG : integer := 30;	-- Common configuration of Bgo trains	   
constant BC_ALIGNMENT_STATUS : integer := 31;  
--constant BGO_TRIGGER_ERROR : integer := 31; -- Common configuration of Bgo trains	   

	 -- Cyclic trigger / Bgo generator Specific Registers	   
constant n_o_regs_per_SCG : integer := 8;	 -- number of regs per single cyclic trig gen	   
constant number_of_ici_cyclic_generators : integer := 8;	 -- 	   
constant STA_CG : integer := 128;	 -- Start of iCi Cyclic generator address space	   
constant END_CG : integer := 191;	 -- End of iCi Cyclic generator address space	   
constant CG_IHB_OFS : integer := 0;	 -- Inhibit reg offset	   
constant CG_IPSC_OFS : integer := 1;	 -- Initial prescale reg offset	   
constant CG_PSC_OFS : integer := 2;	 -- prescale reg offset	   
constant CG_POS_OFS : integer := 3;	 -- postscale reg offset	   
constant CG_PAS_OFS : integer := 4;	 -- Pause reg offset	   
constant CG_TGV_OFS : integer := 5;	 -- trigger value (b-go command)	   
constant CG_dummy1_OFS : integer := 6;	 -- trigger value (b-go command)	   
constant CG_dummy2_OFS : integer := 7;	 -- trigger value (b-go command)	   
		   
constant STA_TRR : integer := 192;	 -- Start of trigger rule checker space
constant TRR_CNT_OFS : integer := 0; -- Trigger Inhibit_counter
-- Offset = 1 houses the deadtime after one trigger, i.e. 2 according to note  02_033.
-- Offset = 2 houses the deadtime after the first of two triggers, i.e. 24 according to note  02_033.
-- Offset = 3 houses the deadtime after the first of three triggers, i.e. 99 according to note  02_033.
-- Offset = 4 houses the deadtime after the first of four triggers, i.e. 239 according to note  02_033.
-- Offset = 5 houses the deadtime after the first of five triggers, i.e. disabled according to note  02_033.
-- Offset = 6 houses the deadtime after the first of six triggers, i.e. disabled according to note  02_033.
-- Offset = 7 houses the deadtime after the first of seven triggers, i.e. disabled according to note  02_033.
-- Offset = 8 houses the deadtime after the first of eight triggers, i.e. disabled according to note  02_033.
-- Offset = 9 houses the deadtime after the first of nine triggers, i.e. 3563 according to note  02_033.
constant END_TRR : integer := 207;	 -- End of trigger rule checker space	   
constant STA_TTS_ENA : integer := 208;	 -- Start of TTS enable space	   
constant END_TTS_ENA : integer := 208 + 8 - 1;	 -- End of TTS enable space	   
constant STA_RETRI : integer := 224;	 -- Start of ReTri space	   
constant END_RETRI : integer := 224 + 32 - 1;	 -- Start of ReTri space	   
	 -- B Channel Specific Registers	   
constant n_o_regs_per_BCH : integer := 4;	 -- number of regs per B-channel	   
constant STA_BCH : integer := 256;	 -- B channel regs start address	   
constant END_BCH : integer := STA_BCH + 16 * n_o_regs_per_BCH - 1;	 -- B channel regs stop address	   
constant BCH_IHB_OFS : integer := 0;	 -- Inhibit reg offset	   
constant BCH_PSC_OFS : integer := 1;	 -- prescale reg offset	   
constant BCH_POS_OFS : integer := 2;	 -- postscale reg offset	   
constant BCH_RMC_OFS : integer := 3;	 -- RMC reg offset	   
constant n_o_regs_per_MBCH : integer := 64;	 -- number of regs per B-channel	   
constant n_o_mbch_blocks : integer := 3;	 -- Number of multi-Bchannel blocks	   
constant n_o_bgos_per_mbch_block : integer := 16;	 -- Number of Bchannel per multi-Bchannel block
constant MBCH_CFG_OFS : integer := 0;	 -- Configuration register
constant MBCH_STA_OFS : integer := 1;	 -- Status register	   
constant MBCH_CNT_STA : integer := 32;	 -- Counter regs start	   
constant MBCH_CNT_STO : integer := 47;	 -- Counter regs stop
constant MBCH_RAM_STA : integer := 48;	 -- RAM start  
constant MBCH_RAM_STO : integer := 63;	 -- RAM stop 

constant STA_TTSSPY : integer := 384;	 -- Start of Selected TTS source spy	   
constant END_TTSSPY : integer := 384 + 32 - 1;	 -- Start of Selected TTS source spy	   	   
		   
constant StartRegs :					unsigned( 31 downto 0) := X"00000000"; -- 0				   
constant StopRegs :					unsigned( 31 downto 0) := X"000001FF"; -- 511		   
constant StartMBCH :					unsigned( 31 downto 0) := X"00000200"; -- 512				   
constant StopBMCH :					unsigned( 31 downto 0) := X"000003FF"; -- 1023		   
constant STA_MBCH :					integer := to_integer(StartMBCH);	 -- Multi B channel regs start address	   
constant END_MBCH :					integer := to_integer(StopBMCH);	 -- Multi B channel regs stop address	   
constant StartTrigDPRAM :			unsigned( 31 downto 0) := X"00000400"; -- 1024		   
constant StopTrigDPRAM :			unsigned( 31 downto 0) := X"000007FF"; -- 2047		   
constant StartBunchTrigDPRAM :	unsigned( 31 downto 0) := X"00000800"; -- 2048		   
constant StopBunchTrigDPRAM :		unsigned( 31 downto 0) := X"000009FF"; -- 3071		   
constant StartBDPRAM :				unsigned( 31 downto 0) := X"00020000"; -- 131072		   8 kword of 32 bits per Bchannel; total 128kword for 16 Bchannels
constant StopBDPRAM :				unsigned( 31 downto 0) := X"0003FFFF"; -- 262143		   
		   
-- Registers per LPM instance
constant SLINK_STAT : integer := 32;	 -- SLINK status (read only)	   
constant SLINK_CONF : integer := 33;	 -- SLINK configuration	   
constant TTS_DELAY : integer := 34;	 -- Trigger throttle value	   
constant TTS_FIFLG : integer := 35;	 -- Trigger throttle fifo flags (read only)	   
constant TTS_FIFO : integer := 36;	 -- Trigger throttle event data fifo (read only)	   



type RegsType is array( 0 to NumberOfRegs - 1)  of std_logic_vector(31 downto 0);		   
type array12x16b is array( 1 to 12) of std_logic_vector(15 downto 0);		   
type array12x32b is array( 1 to 12) of std_logic_vector(31 downto 0);		   
type array16x32b is array( 0 to 15) of std_logic_vector(31 downto 0);		   
type array32x32b is array( 0 to 31) of std_logic_vector(31 downto 0);		   
constant array32x32b_init : array32x32b := (
	x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", 
	x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", 
	x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", 
	x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000", x"00000000");


type tts_state is (Disconnected, Error, Ready, WarningOverflow, Busy, OutOfSynch);

--type bgo_channel_config is
--	record 
--		permanent : std_logic;
--		firing_bc : std_logic_vector(11 downto 0);
--		initial_prescale : integer range 0 to 65535;
--		prescale : integer range 0 to 65535;
--		postscale : integer range 0 to 65535;
--	end record;

--constant BGO_CH_FBC_OFS : integer := 0;	 -- firing_bc reg offset	   
--constant BGO_CH_IPSC_OFS : integer := 1;	 -- Initial prescale reg offset	   
--constant BGO_CH_PSC_OFS : integer := 2;	 -- prescale reg offset	   
--constant BGO_CH_POS_OFS : integer := 3;	 -- postscale reg offset	   

type lumidaq_header_record_type is array (0 to 15) of std_logic_vector(31 downto 0);
-- Attribution as of 20140123:
-- word 0: reserved
-- word 1: nibble number
-- word 2: section number
-- word 3: run number
-- word 4: LHC fill number
-- word 5 to 15: reserved

constant bgo_train_id_start_bit: integer := 24;
constant bgo_train_id_no_bits: integer := 6;
constant permanent_bit: integer := 16;
constant bgo_ch_end_bit: integer := 17;
constant firing_bc_start_bit: integer := 0;
constant firing_bc_n_o_bits: integer := 12;

constant BGO_TR_FBC_OFS : integer := 0;	 -- firing_bc reg offset	   
constant BGO_TR_IPSC_OFS : integer := 1;	 -- Initial prescale reg offset	   
constant BGO_TR_PSC_OFS : integer := 2;	 -- prescale reg offset	   
constant BGO_TR_POS_OFS : integer := 3;	 -- postscale reg offset	   

--constant StartRegsTriggerRules :						integer := 128; -- need 16 registers
--constant StopRegsTriggerRules :						integer := StartRegsTriggerRules + 15;				   

constant number_of_cyclic_generators :			integer := 16;
constant max_number_of_cyclic_generators :	integer := 32;
constant cyclic_gen_counter_width :				integer := 16;
constant StartRegsCyclicGenerator :				integer := 256; -- need 8 registers per cyclic generator
constant StopRegsCyclicGenerator :				integer := StartRegsCyclicGenerator + max_number_of_cyclic_generators * 8 - 1;	-- 511		   

constant n_o_bgo_channels : integer := n_o_bgos;		   
constant max_n_o_bgo_channels : integer := 64;		   
--constant n_o_regs_per_bgo_channel : integer := 4;																						-- Bgo channel address space	   
constant StartRegsBgoChannels :							integer := 512; -- need 1 register per Bgo channel for firing_bc (low 12 bits) and one for Bgo counter.
constant StopRegsBgoChannels :							integer := StartRegsBgoChannels + 2 * max_n_o_bgo_channels - 1;

constant number_of_tts_based_trigger_generators :		integer := 8;
constant max_number_of_tts_based_trigger_generators : integer := 31;
constant number_of_regs_per_tts_based_trigger_generator : integer := 32; -- 5 bits
constant StartRegsTtsBasedTriggerGenerator :				integer := 1024; -- need 32 registers per cyclic generator == 5 address bits
constant StopRegsTtsBasedTriggerGenerator :				integer := StartRegsTtsBasedTriggerGenerator + (max_number_of_tts_based_trigger_generators + 1) * 32 - 1; -- 2047		   

constant StartRegsBunchTriggerGenerator :				integer := 2048; -- need 512 registers == 9 address bits
constant StopRegsBunchTriggerGenerator :				integer := 3071;		   

constant n_o_bgo_train_definitions : integer := 32;		   
constant max_n_o_bgo_train_definitions : integer := 64;		   
constant n_o_regs_per_bgo_train : integer := 128;																						-- Bgo trains address space	   
constant StartRegsTrainBuilderStatus : integer := 8192;																						-- Start of Bgo trains address space	   
constant StopRegsTrainBuilderStatus : integer := 16383;																						-- Start of Bgo trains address space	   
constant StartRegsTrainBuilder :						integer := 16384;				   
constant StopRegsTrainBuilder :						integer := StartRegsTrainBuilder + max_n_o_bgo_train_definitions * n_o_regs_per_bgo_train - 1;		   
constant StartRegsTriggerHistos :					integer := 65536;				   
constant StopRegsTriggerHistos :						integer := StartRegsTriggerHistos + 65536 - 1;		   

--max_bgoch_error: assert not(n_o_bgo_channels > max_n_o_bgo_channels) severity failure;

--constant STA_BGO_OVERLAP_COUNTERS : integer := 64;																						-- Start of Bgo overlap address space	   
--constant END_BGO_OVERLAP_COUNTERS : integer := STA_BGO_OVERLAP_COUNTERS + n_o_bgo_channels - 1;							-- End of Bgo overlap address space	   
--constant STA_BGO_CH : integer := 256;																										-- Start of Bgo channels address space	   
--constant END_BGO_CH : integer := 256 + n_o_bgo_channels * n_o_regs_per_bgo_channel - 1;									-- End of Bgo channels address space	   
--constant STA_BGO_TRAIN_CH : integer := 16384;																							-- Start of Bgo trains address space	   
--constant END_BGO_TRAIN_CH : integer := STA_BGO_TRAIN_CH + n_o_bgo_train_definitions * n_o_regs_per_bgo_train - 1;	-- End of Bgo trains address space	   

end tcds_types;		 
